import logging
from typing import Optional
import ctypes
from . import util

class KrillCore:
    """
    Calls Zig-made shared libraries via ctypes and
    provides core functionality to search for key→subkey from JSON.
    """
    def __init__(self, json_path: str, key: str, subkey: str):
        lib_path = util.get_liblogger_path()
        lib = ctypes.CDLL(lib_path)
        self._lib = lib
        self._lib.find.argtypes = [
            ctypes.POINTER(ctypes.c_ubyte), ctypes.c_size_t,  # json_ptr, json_len
            ctypes.POINTER(ctypes.c_ubyte), ctypes.c_size_t,  # key_ptr, key_len
            ctypes.POINTER(ctypes.c_ubyte), ctypes.c_size_t,  # subkey_ptr, subkey_len
            ctypes.POINTER(ctypes.c_ubyte), ctypes.c_size_t   # out_buf, out_buf_len
        ]
        self._lib.find.restype = ctypes.c_bool

        self.json_path = json_path
        self.key = key
        self.subkey = subkey
        self.buf_size = 256

    def setSubkey(self, subkey: str):
        self.subkey = subkey

    def _bytes_from_string(self, s: str) -> bytes:
        return s.encode('utf-8')

    def find(self) -> Optional[str]:
        key_b = self._bytes_from_string(self.key)
        subkey_b = self._bytes_from_string(self.subkey)

        with open(self.json_path, 'rb') as f:
            json_bytes = f.read()

        json_arr   = (ctypes.c_ubyte * len(json_bytes)).from_buffer_copy(json_bytes)
        key_arr    = (ctypes.c_ubyte * len(key_b)).from_buffer_copy(key_b)
        subkey_arr = (ctypes.c_ubyte * len(subkey_b)).from_buffer_copy(subkey_b)
        out_buf    = (ctypes.c_ubyte * self.buf_size)()

        found = self._lib.find(
            json_arr,      len(json_bytes),
            key_arr,       len(key_b),
            subkey_arr,    len(subkey_b),
            out_buf,       self.buf_size
        )

        if not found:
            return None

        raw = bytes(out_buf)
        return raw.split(b'\0', 1)[0].decode('utf-8', errors='replace')


class KrillLogger(logging.Logger):
    """
    A custom logger that integrates with KrillCore to log messages
    with optional subkey tracking.
    """
    def __init__(self, name: str, level: int = logging.NOTSET):
        super().__init__(name, level)
        self._krill: Optional[KrillCore] = None
        self.alloc_id_list: list[Optional[str]] = []

    def getSubKeyList(self) -> list[Optional[str]]:
        return self.alloc_id_list

    def getLastSubkey(self) -> Optional[str]:
        return self.alloc_id_list[-1] if self.alloc_id_list else None

    def _log_and_record(self, level_fn, msg, identifier: bool, *args, **kwargs):
        level_fn(msg, *args, **kwargs)
        if identifier and self._krill is not None:
            self._krill.setSubkey(msg)
            self.alloc_id_list.append(self._krill.find())

    def debug(self, msg, identifier: bool = False, *args, **kwargs):
        self._log_and_record(super().debug, msg, identifier, *args, **kwargs)

    def info(self, msg, identifier: bool = False, *args, **kwargs):
        self._log_and_record(super().info, msg, identifier, *args, **kwargs)

    def warning(self, msg, identifier: bool = False, *args, **kwargs):
        self._log_and_record(super().warning, msg, identifier, *args, **kwargs)

    def error(self, msg, identifier: bool = False, *args, **kwargs):
        self._log_and_record(super().error, msg, identifier, *args, **kwargs)

    def critical(self, msg, identifier: bool = False, *args, **kwargs):
        self._log_and_record(super().critical, msg, identifier, *args, **kwargs)


# def setup_logger(name: str, json_path: str, level: int = logging.INFO) -> logging.Logger:
#     logging.setLoggerClass(KrillLogger)

#     logger = logging.getLogger(name)
#     logger.setLevel(level)

#     ch = logging.StreamHandler()
#     ch.setLevel(level)
#     formatter = logging.Formatter('%(asctime)s [%(name)s][%(levelname)s] %(message)s')
#     ch.setFormatter(formatter)
#     logger.addHandler(ch)

#     logger._krill = KrillCore(
#         json_path=json_path,
#         key=name,
#         subkey=""
#     )

#     return logger


# if __name__ == "__main__":
#     API_NAME  = 'SKIC05008E004'
#     JSON_PATH = 'logger_output.json'

#     logger = setup_logger(API_NAME, JSON_PATH)

#     logger.info("hello", identifier=True)
#     logger.debug("debug message", identifier=True)
#     logger.warning("warning message", identifier=True)
#     logger.error("error message", identifier=True)
#     logger.critical("critical message", identifier=True)

#     print("SubKey 一覧:", logger.getSubKeyList())
#     print("最後の SubKey:", logger.getLastSubkey())



import importlib.resources
import sys

def get_liblogger_path():
    # Determine the correct file extension based on the platform
    if sys.platform.startswith("linux"):
        ext = ".so"
    elif sys.platform == "darwin":
        ext = ".dylib"
    elif sys.platform in ("win32", "cygwin"):
        ext = ".dll"
    else:
        raise RuntimeError(f"Unsupported platform: {sys.platform}")
    # Use importlib.resources to find the library file in the c_krill.lib package
    with importlib.resources.path("c_krill.lib", f"liblogger{ext}") as p:
        return str(p)



import os
import logging
import tempfile
import shutil

from c_krill.core import setup_logger

def test_krill_logger():
    root_id = "API_ROOT_ID"
    json_path = "./core/dist/logger_output.json"

    logger = setup_logger(root_id, str(json_path))

    logger.info("info message", identifier=True)
    logger.debug("debug message", identifier=True)
    logger.warning("warning message", identifier=True)
    logger.error("error message", identifier=True)
    logger.critical("critical message", identifier=True)

    expected = ["001", "002", "003", "004", "005"]
    assert logger.getSubKeyList() == expected
    assert logger.getLastSubkey() == "005"


import os
from generate_json import generate_json

def test_generate_json_creates_file():

    pyfile = "core/test/logger.py"
    root_id = "API_ROOT_ID"
    output_path = "./core/dist/logger_output.json"
    env_identifier = True

    generate_json(str(pyfile), root_id, str(output_path), env_identifier)

    assert output_path.exists(), "Output file was not created"

    content = output_path.read_text()
    assert len(content) > 0, "Output file is empty"


import ctypes
from . import util

def generate_json(pyfile: str, root_id: str, output_path: str, env_identifier: bool):
    lib_path = util.get_liblogger_path()
    _lib = ctypes.CDLL(lib_path)
    _lib.toJson.argtypes = [
        ctypes.c_char_p, ctypes.c_size_t,  # pyfile_ptr, pyfile_len
        ctypes.c_char_p, ctypes.c_size_t,  # root_id_ptr, root_id_len
        ctypes.c_char_p, ctypes.c_size_t   # output_path_ptr, output_path_len
    ]
    _lib.toJson.restype = None

    pyfile = _bytes_from_string(pyfile)
    root_id = _bytes_from_string(root_id)
    output_path = _bytes_from_string(output_path)

    _lib.toJson(
        pyfile, len(pyfile),
        root_id, len(root_id),
        output_path, len(output_path),
        env_identifier, env_identifier,
    )

def _bytes_from_string(s: str) -> bytes:
    return s.encode('utf-8')